@@ -15,95 +15,179 @@
 import os
 import subprocess
 import sys
+import warnings
 
+from cliff import command
 import six
 import subunit
 import testtools
 
 from stestr import bisect_tests
 from stestr.commands import load
+from stestr.commands import slowest
 from stestr import config_file
 from stestr import output
 from stestr.repository import abstract as repository
 from stestr.repository import util
 from stestr.testlist import parse_list
+from stestr import user_config
 
 
-def set_cli_opts(parser):
-    parser.add_argument("--failing", action="store_true",
-                        default=False,
-                        help="Run only tests known to be failing.")
-    parser.add_argument("--serial", action="store_true",
-                        default=False,
-                        help="Run tests in a serial process.")
-    parser.add_argument("--concurrency", action="store", default=0,
-                        help="How many processes to use. The default (0) "
-                             "autodetects your CPU count.")
-    parser.add_argument("--load-list", default=None,
-                        help="Only run tests listed in the named file."),
-    parser.add_argument("--partial", action="store_true", default=False,
-                        help="Only some tests will be run. Implied by "
-                             "--failing.")
-    parser.add_argument("--subunit", action="store_true", default=False,
-                        help="Display results in subunit format.")
-    parser.add_argument("--until-failure", action="store_true", default=False,
-                        help="Repeat the run again and again until failure "
-                             "occurs.")
-    parser.add_argument("--analyze-isolation", action="store_true",
-                        default=False,
-                        help="Search the last test run for 2-test test "
-                             "isolation interactions.")
-    parser.add_argument("--isolated", action="store_true",
-                        default=False,
-                        help="Run each test id in a separate test runner.")
-    parser.add_argument("--worker-file", action="store", default=None,
-                        dest='worker_path',
-                        help="Optional path of a manual worker grouping file "
-                             "to use for the run")
-    parser.add_argument('--blacklist-file', '-b',
-                        default=None, dest='blacklist_file',
-                        help='Path to a blacklist file, this file '
-                             'contains a separate regex exclude on each '
-                             'newline')
-    parser.add_argument('--whitelist-file', '-w',
-                        default=None, dest='whitelist_file',
-                        help='Path to a whitelist file, this file '
-                             'contains a separate regex on each newline.')
-    parser.add_argument('--black-regex', '-B', default=None,
-                        dest='black_regex',
-                        help='Test rejection regex. If a test cases name '
-                        'matches on re.search() operation , '
-                        'it will be removed from the final test list. '
-                        'Effectively the black-regexp is added to '
-                        ' black regexp list, but you do need to edit a file. '
-                        'The black filtering happens after the initial '
-                        ' white selection, which by default is everything.')
-    parser.add_argument('--no-discover', '-n', default=None, metavar='TEST_ID',
-                        help="Takes in a single test to bypasses test discover"
-                             " and just execute the test specified. A file "
-                             "name may be used in place of a test name.")
-    parser.add_argument('--random', '-r', action="store_true", default=False,
-                        help="Randomize the test order after they are "
-                             "partitioned into separate workers")
-    parser.add_argument('--combine', action='store_true', default=False,
-                        help="Combine the results from the test run with the "
-                             "last run in the repository")
-    parser.add_argument('--no-subunit-trace', action='store_true',
-                        default=False,
-                        help='Disable the default subunit-trace output filter')
-    parser.add_argument('--color', action='store_true', default=False,
-                        help='Enable color output in the subunit-trace output,'
-                             ' if subunit-trace output is enabled. (this is '
-                             'the default). If subunit-trace is disable this '
-                             ' does nothing.')
-    parser.add_argument('--abbreviate', action='store_true',
-                        dest='abbreviate',
-                        help='Print one character status for each test')
+class Run(command.Command):
+    def get_parser(self, prog_name):
+        parser = super(Run, self).get_parser(prog_name)
+        parser.add_argument("filters", nargs="*", default=None,
+                            help="A list of string regex filters to initially "
+                            "apply on the test list. Tests that match any of "
+                            "the regexes will be used. (assuming any other "
+                            "filtering specified also uses it)")
+        parser.add_argument("--failing", action="store_true",
+                            default=False,
+                            help="Run only tests known to be failing.")
+        parser.add_argument("--serial", action="store_true",
+                            default=False,
+                            help="Run tests in a serial process.")
+        parser.add_argument("--concurrency", action="store", default=None,
+                            type=int,
+                            help="How many processes to use. The default (0) "
+                            "autodetects your CPU count.")
+        parser.add_argument("--load-list", default=None,
+                            help="Only run tests listed in the named file."),
+        parser.add_argument("--partial", action="store_true", default=False,
+                            help="DEPRECATED: Only some tests will be run. "
+                            "Implied by --failing. This option is deprecated "
+                            "and no longer does anything. It will be removed "
+                            "in the future")
+        parser.add_argument("--subunit", action="store_true", default=False,
+                            help="Display results in subunit format.")
+        parser.add_argument("--until-failure", action="store_true",
+                            default=False,
+                            help="Repeat the run again and again until "
+                            "failure occurs.")
+        parser.add_argument("--analyze-isolation", action="store_true",
+                            default=False,
+                            help="Search the last test run for 2-test test "
+                            "isolation interactions.")
+        parser.add_argument("--isolated", action="store_true",
+                            default=False,
+                            help="Run each test id in a separate test runner.")
+        parser.add_argument("--worker-file", action="store", default=None,
+                            dest='worker_path',
+                            help="Optional path of a manual worker grouping "
+                            "file to use for the run")
+        parser.add_argument('--blacklist-file', '-b',
+                            default=None, dest='blacklist_file',
+                            help='Path to a blacklist file, this file '
+                            'contains a separate regex exclude on each '
+                            'newline')
+        parser.add_argument('--whitelist-file', '-w',
+                            default=None, dest='whitelist_file',
+                            help='Path to a whitelist file, this file '
+                            'contains a separate regex on each newline.')
+        parser.add_argument('--black-regex', '-B', default=None,
+                            dest='black_regex',
+                            help='Test rejection regex. If a test cases name '
+                            'matches on re.search() operation , '
+                            'it will be removed from the final test list. '
+                            'Effectively the black-regexp is added to '
+                            ' black regexp list, but you do need to edit a '
+                            'file. The black filtering happens after the '
+                            'initial white selection, which by default is '
+                            'everything.')
+        parser.add_argument('--no-discover', '-n', default=None,
+                            metavar='TEST_ID',
+                            help="Takes in a single test to bypasses test "
+                            "discover and just execute the test specified. A "
+                            "file may be used in place of a test name.")
+        parser.add_argument('--random', '-r', action="store_true",
+                            default=False,
+                            help="Randomize the test order after they are "
+                            "partitioned into separate workers")
+        parser.add_argument('--combine', action='store_true', default=False,
+                            help="Combine the results from the test run with "
+                            "the last run in the repository")
+        parser.add_argument('--no-subunit-trace', action='store_true',
+                            default=False,
+                            help='Disable the default subunit-trace output '
+                            'filter')
+        parser.add_argument('--force-subunit-trace', action='store_true',
+                            default=False,
+                            help='Force subunit-trace output regardless of any'
+                                 'other options or config settings')
+        parser.add_argument('--color', action='store_true', default=False,
+                            help='Enable color output in the subunit-trace '
+                            'output, if subunit-trace output is enabled. '
+                            '(this is the default). If subunit-trace is '
+                            'disable this does nothing.')
+        parser.add_argument('--slowest', action='store_true', default=False,
+                            help='After the test run, print the slowest '
+                            'tests.')
+        parser.add_argument('--abbreviate', action='store_true',
+                            dest='abbreviate',
+                            help='Print one character status for each test')
+        return parser
 
+    def get_description(self):
+        help_str = "Run the tests for a project and load them into a "
+        "repository."
+        return help_str
 
-def get_cli_help():
-    help_str = "Run the tests for a project and load them into a repository."
-    return help_str
+    def take_action(self, parsed_args):
+        user_conf = user_config.get_user_config(self.app_args.user_config)
+        filters = parsed_args.filters
+        args = parsed_args
+        if getattr(user_conf, 'run', False):
+            if not user_conf.run.get('no-subunit-trace'):
+                if not args.no_subunit_trace:
+                    pretty_out = True
+                else:
+                    pretty_out = False
+            else:
+                pretty_out = False
+
+            pretty_out = args.force_subunit_trace or pretty_out
+            if args.concurrency is None:
+                concurrency = user_conf.run.get('concurrency', 0)
+            else:
+                concurrency = args.concurrency
+            random = args.random or user_conf.run.get('random', False)
+            color = args.color or user_conf.run.get('color', False)
+            abbreviate = args.abbreviate = user_conf.run.get(
+                'abbreviate', False)
+        else:
+            pretty_out = args.force_subunit_trace or not args.no_subunit_trace
+            concurrency = args.concurrency or 0
+            random = args.random
+            color = args.color
+            abbreviate = args.abbreviate
+        verbose_level = self.app.options.verbose_level
+        stdout = open(os.devnull, 'w') if verbose_level == 0 else sys.stdout
+        result = run_command(
+            config=self.app_args.config, repo_type=self.app_args.repo_type,
+            repo_url=self.app_args.repo_url,
+            test_path=self.app_args.test_path, top_dir=self.app_args.top_dir,
+            group_regex=self.app_args.group_regex, failing=args.failing,
+            serial=args.serial, concurrency=concurrency,
+            load_list=args.load_list, partial=args.partial,
+            subunit_out=args.subunit, until_failure=args.until_failure,
+            analyze_isolation=args.analyze_isolation, isolated=args.isolated,
+            worker_path=args.worker_path, blacklist_file=args.blacklist_file,
+            whitelist_file=args.whitelist_file, black_regex=args.black_regex,
+            no_discover=args.no_discover, random=random,
+            combine=args.combine,
+            filters=filters, pretty_out=pretty_out, color=color,
+            stdout=stdout, abbreviate=abbreviate)
+
+        # Always output slowest test info if requested, regardless of other
+        # test run options
+        user_slowest = False
+        if getattr(user_conf, 'run', False):
+            user_slowest = user_conf.run.get('slowest', False)
+        if args.slowest or user_slowest:
+            slowest.slowest(repo_type=self.app_args.repo_type,
+                            repo_url=self.app_args.repo_url)
+
+        return result
 
 
 def _find_failing(repo):
@@ -159,7 +243,9 @@ def run_command(config='.stestr.conf', repo_type='file',
         autodetects your CPU count and uses that.
     :param str load_list: The path to a list of test_ids. If specified only
         tests listed in the named file will be run.
-    :param bool partial: Only some tests will be run. Implied by `--failing`.
+    :param bool partial: DEPRECATED: Only some tests will be run. Implied by
+        `--failing`. This flag is deprecated because and doesn't do anything
+        it will be removed in a future release.
     :param bool subunit_out: Display results in subunit format.
     :param bool until_failure: Repeat the run again and again until failure
         occurs.
@@ -194,6 +280,9 @@ def run_command(config='.stestr.conf', repo_type='file',
         for failures.
     :rtype: int
     """
+    if partial:
+        warnings.warn('The partial flag is deprecated and has no effect '
+                      'anymore')
     try:
         repo = util.get_repo_open(repo_type, repo_url)
     # If a repo is not found, and there a testr config exists just create it
@@ -221,7 +310,7 @@ def run_command(config='.stestr.conf', repo_type='file',
                 subprocess.Popen(run_cmd, shell=True,
                                  stdout=subprocess.PIPE)))]
             return load.load(in_streams=run_proc,
-                             partial=partial, subunit_out=subunit_out,
+                             subunit_out=subunit_out,
                              repo_type=repo_type,
                              repo_url=repo_url, run_id=combine_id,
                              pretty_out=pretty_out,
@@ -319,9 +408,6 @@ def run_command(config='.stestr.conf', repo_type='file',
                               abbreviate=abbreviate)
     else:
         # Where do we source data about the cause of conflicts.
-        # XXX: Should instead capture the run id in with the failing test
-        # data so that we can deal with failures split across many partial
-        # runs.
         latest_run = repo.get_latest_run()
         # Stage one: reduce the list of failing tests (possibly further
         # reduced by testfilters) to eliminate fails-on-own tests.
@@ -372,14 +458,11 @@ def _run_tests(cmd, failing, analyze_isolation, isolated, until_failure,
             run_procs = [('subunit',
                           output.ReturnCodeToSubunit(
                               proc)) for proc in cmd.run_tests()]
-            partial = False
-            if (failing or analyze_isolation or isolated):
-                partial = True
             if not run_procs:
                 stdout.write("The specified regex doesn't match with anything")
                 return 1
             return load.load((None, None), in_streams=run_procs,
-                             partial=partial, subunit_out=subunit_out,
+                             subunit_out=subunit_out,
                              repo_type=repo_type,
                              repo_url=repo_url, run_id=combine_id,
                              pretty_out=pretty_out, color=color, stdout=stdout,
@@ -409,23 +492,3 @@ def _run_tests(cmd, failing, analyze_isolation, isolated, until_failure,
                     return result
     finally:
         cmd.cleanUp()
-
-
-def run(arguments):
-    filters = arguments[1] or None
-    args = arguments[0]
-    pretty_out = not args.no_subunit_trace
-
-    return run_command(
-        config=args.config, repo_type=args.repo_type, repo_url=args.repo_url,
-        test_path=args.test_path, top_dir=args.top_dir,
-        group_regex=args.group_regex, failing=args.failing, serial=args.serial,
-        concurrency=args.concurrency, load_list=args.load_list,
-        partial=args.partial, subunit_out=args.subunit,
-        until_failure=args.until_failure,
-        analyze_isolation=args.analyze_isolation, isolated=args.isolated,
-        worker_path=args.worker_path, blacklist_file=args.blacklist_file,
-        whitelist_file=args.whitelist_file, black_regex=args.black_regex,
-        no_discover=args.no_discover, random=args.random, combine=args.combine,
-        filters=filters, pretty_out=pretty_out, color=args.color,
-        abbreviate=args.abbreviate)
